@@ -1,9 +1,13 @@
 #!/usr/bin/python
 # -*- coding: utf-8 -*-
 
-import os, os.path, math, fnmatch, re, time, calendar, string, socket, sys, md5, random, ConfigParser, socket
+VERSION = "0.6.3"
+VERDATE = "1 Sep 2008"
+
+import os, os.path, math, fnmatch, re, time, string, socket, sys, md5, random, ConfigParser, socket, urllib
 from copy import copy
 from sets import Set
+
 geocoder = None
 try:
     import GeoIP
@@ -11,6 +15,7 @@ try:
 except:
     pass
 
+stdscr = None
 
 LOG_LEVEL          = 1        # 0 == quiet, 1 == normal, 2 == debug
 DISC_SYNC_CNT      = 100000   # number of records to hold in memory before flushing to disk
@@ -33,14 +38,16 @@ LOG_ROOT = LOG_FILE = LOG_FILE_TYPE = LOG_FORMAT = DEFAULT_OUTPUT_FIELDS = MAX_R
 config = {}
 
 
+def cfg_home():
+    if (os.name != 'posix'): return sys.prefix
+    return '/etc'
+
 # yes, this is ugly.
 def configure(cfg_file=None):
     global config, LOG_ROOT, LOG_FILE, LOG_FILE_TYPE, LOG_FORMAT, DEFAULT_OUTPUT_FIELDS, MAX_REQUEST_TIME, MIN_RPS,re_robots,re_generic,re_classes,LOG_PATTERN,LOG_COLUMNS
 
     if not cfg_file:
-        cfg_file    = '/etc/wtop.cfg'                              # unix, osx, etc
-        if os.name != 'posix' and os.environ.has_key('HOMEPATH'):  # windows
-            cfg_file = os.path.join(os.environ.get('HOMEPATH'), 'wtop.cfg')
+        cfg_file = os.path.join(cfg_home(), 'wtop.cfg')
 
     if not os.path.exists(cfg_file):
         raise Exception("can't read config file: '%s'" % cfg_file)
@@ -48,9 +55,9 @@ def configure(cfg_file=None):
     config = ConfigParser.ConfigParser()
     config.read(cfg_file)
 
-    LOG_ROOT               = config.get('main', 'log_root')
-    LOG_FILE               = config.get('main', 'log_file')
     LOG_FILE_TYPE          = config.get('main', 'log_file_type')
+
+    LOG_ROOT               = config.get('main', 'log_root')
     LOG_FORMAT             = config.get('main', 'log_format')
     DEFAULT_OUTPUT_FIELDS  = config.get('main', 'default_output_fields').split(',')
     MAX_REQUEST_TIME       = int(config.get('wtop', 'max_request_time'))
@@ -65,6 +72,7 @@ def configure(cfg_file=None):
     if LOG_FILE_TYPE == 'apache':
         # these may be overridden later by the logrep command line program because it
         # knows the fields the user asked for.
+        LOG_FILE                = config.get('main', 'log_file')
         LOG_PATTERN,LOG_COLUMNS = format2regexp(config.get('main', 'log_format'))
 
 
@@ -87,8 +95,8 @@ requoted_skipped = r'[^"]*'
 
 # {DIRECTIVE_SYMBOL: (FIELD_NAME, REGEX_WHEN_NEEDED, REGEX_WHEN_SKIPPED)}
 LOG_DIRECTIVES = {
-    'h' : ('ip',         restr, restr_skipped),  # rhost is an odd one, since the default Apache is %h
-    'a' : ('ip',         restr, restr_skipped),  # but HostnameLookups changes its content. bleh.
+    'h' : ('ip',         restr, restr_skipped),  # ip is an odd one, since the default Apache is %h but
+    'a' : ('ip',         restr, restr_skipped),  # HostnameLookups changes its content and %a is ALSO the ip. bleh.
     'l' : ('auth',       restr, restr_skipped),
     'u' : ('username',   restr, restr_skipped),
     't' : ('timestamp',  r'\[?(\S+(?:\s+[\-\+]\d+)?)\]?', r'\[?\S+(?:\s+[\-\+]\d+)?\]?'),
@@ -224,18 +232,17 @@ def domain(url):
     if m: return m.group(1)
     return url
 
+# accepts host or IP.
+def geocode_country(host):
+    if reip.match(host):
+        return geocoder.country_name_by_addr(host)
+    return geocoder.country_name_by_name(host)
+
+def geocode_cc(host):
+    if reip.match(host):
+        return geocoder.country_code_by_addr(host)
+    return geocoder.country_code_by_name(host)
 
-# todo.. the ':' thing is a defense against ipv6 addresses
-# also we're kind of relying on the OS to cache ip resolutions
-ipcache = {}
-def lookup_ip(rhost):
-    if not ipcache.has_key(rhost):
-        if (reip.match(rhost) or rhost[0] == ':'): return rhost
-        try:
-            ipcache[rhost] = socket.gethostbyname(rhost)
-        except:
-            ipcache[rhost] = rhost
-    return ipcache[rhost]
 
 # field massage & mapping to derived fields
 # SOURCE-FIELD, (DERIVED-FIELDS), FUNCTION
@@ -257,13 +264,8 @@ col_fns = [
 # only possible if the geocoding lib is loaded. 
 # hack: this does NOT work if HostnameLookups is on.
 if geocoder: 
-    col_fns.append(('ip', ('country',), (lambda ip: str(geocoder.country_name_by_addr(ip)))))
-    col_fns.append(('ip', ('cc',),      (lambda ip: str(geocoder.country_code_by_addr(ip)))))
-
-def hostname_hack():
-    global col_fns
-    col_fns = [('ip', ('ip',), lookup_ip)] + col_fns
-
+    col_fns.append(('ip', ('country',), (lambda ip: str(geocode_country(ip)))))
+    col_fns.append(('ip', ('cc',),      (lambda ip: str(geocode_cc(ip)))))
 
 def listify(x):
     if not hasattr(x,'__iter__'): return (x,)
@@ -287,11 +289,8 @@ def field_dependencies(requested_fields):
     deps = Set(requested_fields)
     deps = Set(flatten(map((lambda f: f[0:2]), filter((lambda f: deps.intersection(f[1])), col_fns))))
     deps = flatten(map((lambda f: f[0:2]), filter((lambda f: deps.intersection(f[1])), col_fns)))
-    return Set(deps + requested_fields)
+    return Set(deps + list(requested_fields))
 
-
-# This, believe it or not, is the work function. It takes in raw log
-# lines and emits a generator of mapped, derived, massaged log records.
 def apache_log(loglines, LOG_PATTERN, LOG_COLUMNS, relevant_fields):
     logpat     = re.compile(LOG_PATTERN) 
     groups     = (logpat.search(line) for line in loglines) 
@@ -303,7 +302,7 @@ def apache_log(loglines, LOG_PATTERN, LOG_COLUMNS, relevant_fields):
 ##########################################################################
 ## IIS-specific stuff. Can't be arsed to libraryize it.
 
-#{'date':'2008-07-21', 'time':'18:09:00'} --> 1216663740
+# {'date':'2008-07-21', 'time':'18:09:00'} --> 1216663740
 def iis2unixtime(r):
     return (((((date_ordinal(int(r['date'][0:4]), int(r['date'][5:7])) + int(r['date'][8:10]))*24) + int(r['time'][0:2]))*60 + int(r['time'][3:5]))*60) + int(r['time'][7:9])
 
@@ -311,7 +310,9 @@ def fix_query(q):
     if q[0] == '-': return ''
     return '?' + q
 
-# source cols, parsing function takes whole record
+# source cols, parsing function takes whole record. makes it hard to 
+# do column dependencies, but with IIS we just split on whitespace
+# anyway.
 iis_col_fns = [
     (('query',),                      (lambda x: fix_query(x['query']))),
     (('url',),                        (lambda x: x['path']+x['query'])),
@@ -328,8 +329,8 @@ iis_col_fns = [
     (('refdom',),                     (lambda x: domain(x['ref']))),
 ]
 if geocoder: 
-    iis_col_fns.append((('country',),     (lambda x: str(geocoder.country_name_by_addr(x['ip'])))))
-    iis_col_fns.append((('cc',),          (lambda x: str(geocoder.country_code_by_addr(x['ip'])))))
+    iis_col_fns.append((('country',),     (lambda x: str(geocode_country(x['ip'])))))
+    iis_col_fns.append((('cc',),          (lambda x: str(geocode_cc(x['ip'])))))
 
 def iis_field_map(log, relevant_fields, col_fns):
     # get only the column functions that are necessary
@@ -343,7 +344,7 @@ def iis_log(loglines, relevant_fields):
     cols = ('date', 'time', 's_sitename', 's_computername', 's_ip', 'method', 'path', 'query', 'port', 'user', 'ip', 
        'proto', 'ua', 'cookie', 'ref', 'vhost', 'status', 'substatus', 'win32_status', 'bytes_in', 'bytes', 'msec')
     tuples     = (line.split(' ') for line in loglines)
-    log        = (dict(zip(cols,t)) for t in tuples) 
+    log        = (dict(zip(cols,urllib.unquote_plus(t))) for t in tuples) 
     log        = iis_field_map(log, relevant_fields, iis_col_fns)
     return log
 ##########################################################################
@@ -600,9 +601,13 @@ def compile_aggregates(commands):
 #
 #  There is implicit conversion of strings that look like numbers. 
 #  There is also support for = and != of multiple values:
-#    
-#  foo=(one,two,three)
-#  foo=one|two|three
+#   
+# todo: what about sets?  
+#  foo=(one,two,three)   or  foo=one|two|three
+#
+# sets can be emulated using a regexp, but it's not always the same result and is
+# probably slower. 
+#  foo~one|two|three
 #
 #
 recmp = re.compile(r'([a-z]+)(>|<|=|\!=|\!~|\~)([^,]+)')
@@ -658,6 +663,7 @@ def tail_n(filename, num):
 def gen_top_stats(reqs, every=5):
     stats = {}
     last_print = 0
+    stats.setdefault('(all)', (rrd2(20000, 30), rrd2(2000, 30), rrd2(200, 30), rrd2(200, 30), rrd2(200, 30)))
 
     for r in reqs:
         # record a hit for the given status class (2xx, 3xx, 4xx, 5xx, slow) to generate rps stats
@@ -665,15 +671,16 @@ def gen_top_stats(reqs, every=5):
 
         if r['msec'] < MAX_REQUEST_TIME:
             stats[r['class']][(r['status']/100)-2].append(1, r['msec']) # 200 = 0, 3xx = 1, 4xx = 2, etc
+            stats['(all)'][(r['status']/100)-2].append(1, r['msec'])
 
         else: # log it in the "slow" bucket
             stats[r['class']][4].append(1, r['msec'])
+            stats['(all)'][4].append(1, r['msec'])
             
         if (time.time() - last_print) > every:
             last_print = time.time()
             yield stats
 
-
 def apache_top_mode(reqs):
     for stats in gen_top_stats(reqs, every=5):
         buf = []
@@ -689,7 +696,15 @@ def apache_top_mode(reqs):
             buf.append('% 34s % 9s % 5d % 4d  %s % 5d % 7s % 7s % 7s % 7s' % (
                     c, pretty_float(rps), avg, mn, sparkline, mx, x3, x4, x5, slow))
 
-        print "\n".join(buf) + "\n\n\n"
+        if stdscr:
+            top_curses_mode(stdscr, buf)
+        else:
+            print "\n".join(buf) + "\n\n\n"
+
+def top_curses_mode(stdscr, buf):
+    stdscr.clear()
+    stdscr.addstr(0, 0, "\n".join(buf))
+    stdscr.refresh()
 
 
 # for both tail and grep mode
